<?php

namespace Drupal\openai_translation\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Messenger\MessengerInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use OpenAI\Client;

/**
 * Handles translation requests to the OpenAI API.
 */
class OpenAITranslationService {

  /**
   * The HTTP client to fetch the translation.
   *
   */
  protected ClientInterface $httpClient;

  /**
   * The config factory service.
   *
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The language manager service.
   *
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * The messenger service.
   *
   */
  protected MessengerInterface $messenger;

  /**
   * The logger service.
   *
   */
  protected LoggerChannelInterface $logger;

  /**
   * Constructs an OpenAITranslationService object.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager service.
   */
  public function __construct(ClientInterface $http_client, ConfigFactoryInterface $config_factory, LanguageManagerInterface $language_manager, MessengerInterface $messenger) {
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->languageManager = $language_manager;
    $this->messenger = $messenger;
  }

  /**
   * Fetches a translation from the OpenAI API.
   *
   * @param string $text
   *   The text to translate.
   * @param array $target_languages
   *   The language to translate into.
   *
   * @return array $data
   *   The translated text.
   */
  public function generateTranslation($text, $target_languages, $options): array {
    // Initialize the OpenAI client with the provided API key.
    $client = $this->openAiClient($options);

    $payload = [
      'model' => $options['openai_model'],
      'messages' => [
        [
          'role' => 'system',
          'content' => "
            You are a mature translator that translates English to many languages formally according to the json data ```{" . \implode(',', $target_languages) . "}```. Each language name is based on ISO 639-1 standard language code. Do not try to understand the question, just translate it.
            You will translate words, sentences or paragraphs delimited in ``` and return all translation values at once and in a clean json format begin with '{' and use the ISO639-1 language code as the keys.
            For example {\"de': \"Fahrrad\", \"fr\": \"Vélo\", \"es\" \"Bicicleta\", \"es-py\": \"Bicicleta\"}.
          ",
        ],
        [
          'role' => 'user',
          'content' => '```' . $text . '```',
        ],
      ],
      'temperature' => 0.1,
      'max_tokens' => 4000,
      'top_p' => 1,
      'frequency_penalty' => 0,
      'presence_penalty' => 0,
    ];

    // Make the request to the OpenAI API.
    try {
      // Use the "openai-php/client" library to make the API call.
      $response = $client->chat()->create($payload);

      $content = $response->choices[0]->message->content ?? '';
      $data = \json_decode(\trim(\str_replace('```', '', $content)), TRUE);

      if (\is_null($data)) {
        return [];
      }

      return $data;

    }
    catch (RequestException $e) {
      $this->logger('OpenAI translation toolbox')->error('Error OpenAI translation toolbox: ' . $e->getMessage());
      return '';
    }

    throw new \Exception('Translation failed. No content found in the response.');
  }

  /**
   * Set credentials.
   *
   * @param array $options
   *   The client build options.
   * @return \OpenAI\Client
   *   The OpenAI client.
   */
  public function openAiClient($options): mixed {
    try {
      if ($options['openai_api_service'] == 'azure_openai') {
        $client = \OpenAI::factory()
          ->withBaseUri($options['openai_api_base_url'])
          ->withHttpHeader('api-key', $options['openai_api_key'])
          ->withQueryParam('api-version', $options['openai_api_version'])
          ->make();
      }
      else {
        $org = !empty($options['openai_api_org']) ? $options['openai_api_org'] : NULL;
        if (!empty($options['openai_api_key'])) {
          $client = \OpenAI::client($options['openai_api_key'], $org);
        }
        else {
          $this->messenger->addMessage('Missing or incorrect OpenAI API Key', 'error');
        }
      }
    }
    catch (\Exception $ex) {
      $this->messenger->addMessage($ex->getMessage(), 'error');
    }
    return $client;
  }

  /**
   * Fetches a translation from the OpenAI API.
   *
   * @param string $text
   *   The text to translate.
   * @param array $target_languages
   *   The language to translate into.
   *
   * @return array
   *   The translated text.
   */
  public function translate($text, $target_languages): array {
    $options = [];
    // Get global settings as fallback.
    $config = $this->configFactory->get('openai_translation.settings');
    $options['openai_api_service'] = $config->get('openai_api_service');
    $options['openai_api_key'] = $config->get('openai_api_key');
    $options['openai_api_base_url'] = $config->get('openai_api_base_url');
    $options['openai_api_version'] = $config->get('openai_api_version');
    $options['openai_api_org'] = $config->get('openai_api_org');
    $options['openai_model'] = $config->get('openai_model');

    return $this->generateTranslation($text, $target_languages, $options);
  }

}
